header
{
/**
 * Lexer for the Mile language.
 * 
 * @author Ricardo Soto
 * @since 1.5
 */

package mileCompiler.compilers;
}

class mileLexer extends Lexer;
options{
    charVocabulary = '\3'..'\377';
    exportVocab=mileLexerVocab;
    testLiterals=false;
    k=2; // lookahead
    defaultErrorHandler=false;
}

tokens
{

//===================================================
// Reserved words
 
   NUMERIC_TYPE  = "numeric";
   STRING_TYPE   = "string" ;

   VAR_RW        = "var"    ;
   BEGIN_RW      = "begin"  ;
   END_RW        = "end"    ;
   IF_RW         = "if"     ;
   ELSE_RW       = "else"   ;
   FOR_RW        = "for"    ;
   OR_RW         = "or"     ;
   AND_RW        = "and"    ;
   NOT_RW        = "not"    ;
   PRINT_RW      = "print"  ;
   READ_RW       = "read"   ;

   TRUE_LITERAL  = "true"   ;
   FALSE_LITERAL = "false"  ;

   INT_LITERAL; 
   REAL_LITERAL;

}


//===================================================
// Letters

   protected LETTER
      : 'a'..'z'
      | 'A'..'Z'
      ;

//===================================================
// Digits

   protected DIGIT : '0'..'9';

//===================================================
// Recognize literals and reserved words

   UNDERSCORE : '_';
   
   IDENT options {testLiterals=true;} // Checking Reserved Words
      : (LETTER) (LETTER|DIGIT|UNDERSCORE)*
      ;


//===================================================
// Punctuation

   SEMICOLON : ';'
             ;
   COMMA     : ','
             ;
   DOT       : '.'
             ;
   COLON     : ':'
             ;

//===================================================
// Brackets

   LEFT_PAREN    : '(' ;
   RIGHT_PAREN   : ')' ;
   LEFT_BRACE    : '{' ;
   RIGHT_BRACE   : '}' ;
   LEFT_BRACKET  : '[' ;
   RIGHT_BRACKET
   options { paraphrase="a right bracket (']')"; }
                 : ']' ;


//==================================================
// Operators

   PLUS  : '+' ;
   SUB   : '-' ;
   STAR  : '*' ;
   SLASH : '/' ;

//==================================================
// Relations

   ASSIG     : '=';
      
//==================================================
// Relations

   EQUAL     : '=''=' ;
   NOT_EQUAL : '!''=' ;
   GR_EQUAL  : '>''=' ;
   LE_EQUAL  : '<''=' ;
   GR        : '>'    ;
   LE        : '<'    ;
   

//===================================================
// Recognize integer literals 

NUMBER : (( DIGIT )+ DOT (DIGIT)+) => ( DIGIT )+ DOT (DIGIT)+ { $setType (REAL_LITERAL);}
         | ( DIGIT )+ { $setType (INT_LITERAL);}
         ;


//==================================================
// White spaces

// \t : tab
// \f : form feed
// \n : newline feed
// \r : carriage return


WS :    ((' ' | '\t' | '\f') 
        | ( '\n' | '\r' ) { newline(); }
        ) { _ttype = Token.SKIP;  }
        ;
        

//====================================================
//String literals

STRING_LITERAL : '"' !
                  ( ~('"'|'\n'|'\r') )*
                  '"' !
                ;
                        
//====================================================
//Comments

LINE_COMMENT :
                "//" ( ~('\n') )* 
                       {_ttype = Token.SKIP;  }
        ;


COMMENT :
	"/*" (options {generateAmbigWarnings=false;}
		:
		{ LA(2)!='/' }? '*'
		|	'\r' '\n'		{newline();}
		|	'\r'			{newline();}
		|	'\n'			{newline();}
		|	~('*'|'\n'|'\r')
		)*
		"*/"
		{$setType(Token.SKIP);}
	;





